#!/usr/bin/perl
# (c) Velytsky, 2005

$0=~ m|(.*/)*(.+)|;
$filename = $+;

$helpinfo = "
This is a perl script that computes a JackKnife average.

Usage:
$filename [-help] [-bin index] [file]

where
\t-help: prints this page
\t-bin index: Number of bins (default -bin #datapoints).
\tfile: read input from the file (default read from stdin).
";
while($ARGV[0] =~ /^-/) {
    $in = shift @ARGV;
    if($in =~ /^-bin$/) {
	die "No number of bins given!" if(!defined $ARGV[0]);
	$bin= shift @ARGV;
    }
    elsif($in =~ /^-help$/) {
	print $helpinfo;
	exit(0);
    }
    else {die "$in is not valid switch"};
}
$i=0;
while($line=<>) {
	@dataIn=split(' ',$line);
	if($i==0) {
		$Ncol=$#dataIn+1;
		$step=$dataIn[0];
	}
	if($i==1) {
		$step=$dataIn[0]-$step;
	}
	for($j=0;$j<$Ncol;$j++) {
		$data[$i][$j]=$dataIn[$j];
	}
	$i++;
}
$Ndata=$#data+1;
if(!defined $bin) {
	$bin=$Ndata;
}
$BinSize=int($Ndata/$bin);
$Nudata=($BinSize)*$bin;
$left= $Ndata-$Nudata;
print STDERR "Bin size $BinSize, $left points not used\n";
for($i=0;$i<$Nudata;$i++) {
	for($j=1;$j<$Ncol;$j++) {
		$sum[$j]+=$data[$i][$j];
	}
}
for($j=1;$j<$Ncol;$j++) {
	$mean[$j]=$sum[$j]/$Nudata;
}
# form bins
open(Bfile,">jackknife.d") or die "Cannot open jackknife.d: $!";
for($i=0;$i<$bin;$i++) {
	$binIndex=($i+1/2)*$BinSize*$step;
#	$binIndex=($i)*$BinSize*$step;
	print Bfile "$binIndex ";
	for($j=1;$j<$Ncol;$j++) {
		for($k=0;$k<$BinSize;$k++) {
			$DataBin[$i][$j]+=$data[$i*$BinSize+$k][$j]/$BinSize;
		}
		print Bfile "$DataBin[$i][$j] ";
	}
	print Bfile "\n";
}
#
for($i=0;$i<$bin;$i++) {
	for($j=1;$j<$Ncol;$j++) {
		$jk_data[$i][$j]=($sum[$j]/$BinSize-$DataBin[$i][$j])/($bin-1);
	}
}
for($i=0;$i<$bin;$i++) {
	for($j=1;$j<$Ncol;$j++) {
		$temp[$j]=$jk_data[$i][$j]-$mean[$j];
		$variance[$j]+=$temp[$j]*$temp[$j];
	}
}
for($j=1;$j<$Ncol;$j++) {
	$variance[$j]*=($bin-1);
	$error[$j]=sqrt($variance[$j]/$bin);
}


print STDERR "Data points: $Ndata, Number of bins $bin\n";
for($j=1;$j<$Ncol;$j++) {
	$r=$j-1;
	print "$r $mean[$j] $error[$j]\n";
}
