#ifndef _SU2XSU2_H_
#define _SU2XSU2_H_
#include <complex>
#include "SU2.h"

class SU2xSU2
{
	friend std::ostream &operator<<( std::ostream & out, const SU2xSU2 & T){
	 	// Print an SU(2)xSU(2) matrix
	 	out<<"[";
	 	for(int i=0;i<4;i++) {
			for(int j=0;j<4;j++) {
				out<<T.get(i,j)<<" ";
			}
			out<<"\n";
		}
		out<<"]";
		return out;
	}
public:
	SU2xSU2() {};
	
	inline void DirectProd(const SU2& lhs, const SU2& rhs) {
		//direct product (U^*)*V
		std::complex<double> I(0,1);
		std::complex<double> LM[2][2];
		std::complex<double> RM[2][2];
		LM[0][0]=lhs[0]+I*lhs[3];
		LM[0][1]=lhs[2]+I*lhs[1];
		LM[1][0]=-lhs[2]+I*lhs[1];
		LM[1][1]=lhs[0]-I*lhs[3];
		RM[0][0]=rhs[0]+I*rhs[3];
		RM[0][1]=rhs[2]+I*rhs[1];
		RM[1][0]=-rhs[2]+I*rhs[1];
		RM[1][1]=rhs[0]-I*rhs[3];
		
		for(int a1=0;a1<2;a1++) {
			for(int a2=0;a2<2;a2++) {
				for(int b1=0;b1<2;b1++) {
					for(int b2=0;b2<2;b2++) {
						data[a1+2*a2][b1+2*b2]=std::conj(LM[a1][a2])*RM[b1][b2];
					}
				}
			}
		}
	}
	inline void operator()(double v) {
		for(int i=0;i<4;i++) {
			for(int j=0;j<4;j++) {
				data[i][j]=0;
			}
			data[i][i]=v;
		}
	}
	inline std::complex<double> get(const int i, const int j) const 
	{ 
		return data[i][j]; 
	}
	inline void operator+=(const SU2xSU2& rhs) {
		for(int i=0;i<4;i++) {
			for(int j=0;j<4;j++) {
				data[i][j]+=rhs.get(i,j);
			}
		}
	}
	inline void operator=(const SU2xSU2& rhs) {
		for(int i=0;i<4;i++) {
			for(int j=0;j<4;j++) {
				data[i][j]=rhs.get(i,j);
			}
		}
	}
	inline void operator*=(const SU2xSU2& rhs) {
		std::complex<double> lhs[4][4];
//		int a,b,g,d;
		for(int i=0;i<4;i++) {
			for(int j=0;j<4;j++) {
				lhs[i][j]=data[i][j];
				data[i][j]=0;
			}
		}
		for(int i=0;i<4;i++) {
			for(int j=0;j<4;j++) {
//				cout<<"i="<<i<<" j="<<j<<" - a="<<i%2<<" b="<<i/2<<" g="<<j%2<<" d="<<j/2<<endl;
				for(int l=0;l<2;l++) {
					for(int e=0;e<2;e++) {
						data[i][j]+=lhs[i%2+2*l][j%2+2*e]*rhs.get(l+2*(i/2),e+2*(j/2));
//						cout<<l<<e<<" "<<i%2+2*l<<j%2+2*e<<l+2*(i/2)<<e+2*(j/2)<<" + ";
					}
				}
//				cout<<endl;
			}
		}
	}
	inline void operator*=(const double rhs) {
		for(int i=0;i<4;i++) {
			for(int j=0;j<4;j++) {
				data[i][j]*=rhs;
			}
		}
	}
	inline double trace() {
		std::complex<double>  tr=0;
		for(int i=0;i<4;i+=3) {
			for(int j=0;j<4;j+=3) {
				tr+=data[i][j];
			}
		}
		return tr.real();
	}
	std::complex<double> Tloop(const SU2& L1, const SU2& L2);
	//~SU2xSU2() {delete [] data;};??????????
private:
	std::complex<double> data[4][4];
};
//==========definitions==============
inline std::complex<double> SU2xSU2::Tloop(const SU2& L1, const SU2& L2)
{
	//form a loop by multiplying by transports L1, L2:
	//loop = L1*T(this)*L2
	std::complex<double> Uproduct(0);
	for(int a1=0;a1<2;a1++) {
		for(int a2=0;a2<2;a2++) {
			for(int b1=0;b1<2;b1++) {
				for(int b2=0;b2<2;b2++) {
					Uproduct+=L1.GetMatrix(a1,b1)*data[a1+2*a2][b1+2*b2]*L2.GetMatrix(a2,b2);//L2 not conjugate
				}
			}
		}
	}
	return Uproduct;
}
#endif //_SU2XSU2_H_
