#ifndef JACK_KNIFE_H
#define JACK_KNIFE_H
// Creates JackKnife bins from input array;
// Calculates from the bins the JackKnife estimators:
// mean value, variance, error bar.

//to add binning for large N !
#include <iostream>
#include <math.h>
using std::cerr;

template< class T >
class JackKnife {
	public:
		JackKnife(T array[], int N=0);
		JackKnife(vector<T> &);
		JackKnife(vector< vector<T> > &);
		inline void compute(T array[], int N=0) {
			if(N==1) {
				mean=array[0];
				variance=0.0; error=0.0;
				return ;
			}
			if(N<1) {
				cerr << "ERROR: no data for the Jack Knife, N="<<N;
				abort();
			}
			T jk_array[N];
			T sum=0.0; 
			mean=0.0,variance=0.0,error=0.0;
			T norm=1.0/static_cast < T > (N-1);
			for(int i=0;i<N;i++) sum+=array[i];
			for(int i=0;i<N;i++) jk_array[i]=norm*(sum-array[i]);
			mean=sum/N;
			{
				T temp;
				for(int i=0;i<N;i++) {
					temp=(jk_array[i]-mean);
					variance+=temp*temp;
				}
			}
			variance*=(N-1);
			error=sqrt(variance/N);
		}
		
		T mn() {return mean;}
		T vr() {return variance;}
		T er() {return error;}
		vector< T > Vmn() {return Vmean;}
		vector< T > Vvr() {return Vvariance;}
		vector< T > Ver() {return Verror;}
	private:
		T mean;
		T variance;
		T error;
		vector< T > Vmean;
		vector< T > Vvariance;
		vector< T > Verror;
};
#endif
