// (c) Alexander Velytsky, 2005
//evolves SU(2) using HB, performs different measurements and decimation
#define CREUTZ
#include "init.h"
#include "lattice.h"
#include "meas.h"
#include "SU2Wilson.h"
#include "SU2Mixed.h"
#include "Demon.h"
#include "OutputManager.h"
//#include "DemonMixed.h"
void VecPrint(ofstream & OutFile, int sweep, vector<double> & Vec);

int main() 
{
	set_new_handler(outOfMemory); //My handler of out of memory conditions
	Params par("parameters.in"); //set parameters from a file
	Params DecPar=par; DecPar.DecReinit(); //decimated lattice parameters
	//--------- Output files -----------------
	//OutputManager MyOutput(par);
	ofstream PlaqFile("Plaq.d", ios::out);   //plaquette average
	ofstream WloopFile("Wloop.d",ios::out); 	//Wilson loops (N x N)
	ofstream PloopFile("Ploop.d",ios::out);  //Polyakov loop average
	ofstream WloopLargeFile("WloopLarge.d",ios::out); //Large Wilson loop (N_tau x r)
	ofstream PloopCorFile("PloopCor.d",ios::out); //Polyakov loop correlator
	ofstream DecPlaqFile("DecPlaq.d",ios::out); //decimated plaquette
	ofstream DecWloopFile("DecWloop.d",ios::out); 
	ofstream PloopCorFileB("PloopCorB.d",ios::out); //before decimation
	ofstream PloopCorFileA("PloopCorA.d",ios::out); //after decimation
	ofstream OutDemonFile("DemEn.d", ios::out); //demon energy
	//---------------------------------------
	Lattice MyLat(par); //declare a lattice (layout only; links stored at sites)
	MyLat.Init(par.start); //init the lattice
	Meas MyMeas(MyLat); //Measurements object (set of algorithms)
	// Evolution class (SU2Wilson/SU2Mixed) defines the evolution algorithm:
	SU2Wilson MyModel(MyLat);
//	SU2Mixed MyModel(MyLat);
	Demon MyDemon(8);
	//-----------------Equilibrization--------------------
	MyModel.Equilibrium(par.getEquil());
	//-----------------Some variables---------------------
	int EvSweeps=par.getEvol();
	int Eskip=1;
	if(EvSweeps>100) Eskip=EvSweeps/100;
	vector<double> DemonEnergy;
	int rMax=par.LatSize[0]/2;
	int MeasSkip=par.getMeasSkip();
	//-----------------Evolution=========================
	cout<<"Evolution: ";
	for (int sweeps=0; sweeps<EvSweeps;sweeps++) {
		if(sweeps%Eskip == 0) cout<<sweeps<<" "<<flush;
		//--------- Evolution algorithm
//		MyModel.EvolveMetro();
		MyModel.Evolve(par.getOverRel()); //for Wilson only
		//MyModel.Evolve(); ///for Mixed
		//--------- Measurements (every MeasSkip sweeps)
		if(sweeps%MeasSkip==0) {
			// -------- Plaquettes ---------
			if(par.DoMeasure(Plaq)) {
				vector<double> PlaqVec(MyMeas.PlaqAll(par.getNrep()));
				VecPrint(PlaqFile,sweeps,PlaqVec);
			}
			// --------- Wilson Loops --------
			if(par.DoMeasure(Wloop)) {
				vector<double> WloopVec(MyMeas.WloopAll(par.getWloopLength()));
				VecPrint(WloopFile,sweeps,WloopVec);
			}
			// -------- Polyakov Loops -------
			if(par.DoMeasure(Ploop)) {
				PloopFile <<sweeps<<" "<<MyMeas.Ploop()<<endl;
			}
			// -------- Polyakov Loop Correlators -------
			if(par.DoMeasure(WloopLarge) || par.DoMeasure(PloopCor)) {
				//		vector<double> PPcor(MyMeas.PloopCor(rMax));//naive
				//Luscher-Weisz multilevel
				if(par.DoMeasure(WloopLarge)) {
					vector<double> Ploop(rMax+1,0), Wloop(rMax+1,0);
					MyMeas.PWloopCorLW(rMax,MyModel, &Ploop, &Wloop);
					VecPrint(WloopLargeFile,sweeps,Wloop);
					if(par.DoMeasure(PloopCor)) VecPrint(PloopCorFile,sweeps,Ploop);
				} else {
					vector<double> Ploop(rMax+1,0);
					MyMeas.PWloopCorLW(rMax,MyModel, &Ploop);
					VecPrint(PloopCorFile,sweeps,Ploop);
				}
			}
			if(par.getDecimation()>0) {
				MyLat.Decimate(par.getDecimation());//Swendsen or Double Smeared Blocking
				Lattice DecLat(DecPar);
				DecLat.Init(MyLat);//take decimated values
				Meas DecMeas(DecLat);
				DecPlaqFile<<sweeps<<" "<<1-DecMeas.Plaq(1)/2<<endl;
				vector<double> DecWloopVec(DecMeas.WloopAll(DecPar.getWloopLength()));
				VecPrint(DecWloopFile,sweeps,DecWloopVec);
	//		}
			// ------- Demon Energy --------
	//		if(par.getMeasDemon()==1) 
	//		{
		//		vector <double> PPcorB(DecMeas.PloopCor(rMax/2));
		//		VecPrint(PloopCorFileB,sweeps,PPcorB);
				MyDemon.Init(DecLat); 
				//MyDemon.Init(MyLat);
			//	if(sweeps==0)
				vector<double> ThermDemonEnergy=MyDemon.En(100);
			/*	if(sweeps==1) {
					for(int i =0;i<100;i++) {*/
				DemonEnergy=MyDemon.En(par.getDemSweeps());
			/*	VecPrint(OutDemonFile,i,DemonEnergy);
					}
				}*/
				VecPrint(OutDemonFile,sweeps,DemonEnergy);
			/*	Lattice DecLat2(DecPar); 
				MyDemon.GetLat(DecLat2);
			//	MyDemon.GetLat(DecLat);
				Meas DecMeas2(DecLat2);
		//		vector <double> PPcorA(DecMeas2.PloopCor(rMax/2));
		//		VecPrint(PloopCorFileA,sweeps,PPcorA);*/
			}
		}
		//-----------
	}
	cout<<EvSweeps<<endl;
	
//	MyLat.Unload("last.d");

	return 1;
}

void VecPrint(ofstream & OutFile, int sweep, vector<double> & Vec) {
	OutFile<<sweep<<" ";
	for(vector<double>::iterator itVec=Vec.begin();itVec!=Vec.end();itVec++) 
		OutFile<<*(itVec)<<" ";
	OutFile<<endl;
}
