#include "Demon.h"
#include <iostream>

Demon::Demon(int Nrep) 
{// This is an SU(2) demon only
	Ed.assign(Nrep,0);
	Lat=0;
}
void Demon::Init(Lattice &MyLat) 
{// This is an SU(2) demon only
	//Init has to be called before the methods can be used!
	if(Lat) 
		GetLat(MyLat);
	else 
		Lat=new Lattice(MyLat);
}

void Demon::GetLat(Lattice &MyLat)
{
	if(MyLat.Vol!=Lat->Vol) {
		cerr<<"Demon::GetLat() Volumes not equal"<<endl;
		exit(-1);
	}
	for (int i=0;i<Lat->Vol;i++) {
		for (int mu=0;mu<Nd;mu++) Lat->Ufield[i][mu]=*MyLat.Umu(i,mu);
	}
}

vector<double> Demon::Action(SU2& Umu, int LinCoord, int mu) 
{//to be taken from somewhere else
	SU2 Vmu;
	vector<double> act;
	act.assign(Lat->Nrep,0);
	for(int nu=0; nu < Nd; nu++) {
		if(nu==mu) continue;
		for (int dir=-1;dir<=1;dir+=2) {
			Vmu=Umu*Lat->Staple(LinCoord,mu,nu,dir);//change to staples!
//decouple tr from trr(1) to make it faster!!
			for(int j=1;j<=Lat->Nrep;j++) 
				act[j-1]+=(-Vmu.Trr(j)/(j+1));
		}
	}
	return act;
}

vector<double> Demon::En(int Nupdates)
{
	SU2 Uprime;
	vector< double > dE;
	vector< double > EdInt(Ed.size(),0);
	int AcceptRate=0;
	int sweeps=Nupdates;
	
	//Creutz type - start with zero energy demon
	//Ed.clear();for(int rep2=1;rep2<=Lat->Nrep;rep2++) Ed.push_back(0);
	if(Nupdates==0) sweeps=1;
	for(int sweep=0;sweep<sweeps;sweep++) {
		for (int LinCoord=0;LinCoord<Lat->Vol;LinCoord++) {
			for (int mu=0;mu<Nd;mu++) {
				Uprime=RMatrix()*(*Lat->Umu(LinCoord,  mu));
				vector<double> ActPrime=Action(Uprime,LinCoord,mu);
				vector<double> Act=Action((*Lat->Umu(LinCoord,mu)),LinCoord,mu);
				for(int rep2=1;rep2<=Lat->Nrep;rep2++) {
					dE.push_back(ActPrime[rep2-1]-Act[rep2-1]);
				}
				bool ltest=true;
				double Emax=5;
				for(int iv=0;iv<Ed.size();iv++) {
					if(!((Ed[iv]-dE[iv]>-Emax) && (Ed[iv]-dE[iv]<Emax))) {
						ltest=false;
						break;
					}
				}
				if (ltest) {//demon has enough energy for an update
					for(int iv=0;iv<Ed.size();iv++) Ed[iv]-=dE[iv];
					AcceptRate++;
					if(Nupdates>0) {	//Update link
						*Lat->Umu(LinCoord,  mu)=Uprime;
					}
				}
				dE.clear();				
				for(int irep=0;irep<Ed.size();irep++) EdInt[irep]+=Ed[irep];
			}
		}
	}
//	cout <<"demon ar:"<<AcceptRate*1.0/(Lat->Vol*Nd*sweeps)<<endl;
	for(int irep=0;irep<Ed.size();irep++) EdInt[irep]/=(Lat->Vol*Nd*sweeps);
	//set Ed to average for the next run
//	for(int irep=0;irep<Ed.size();irep++) Ed[irep]=EdInt[irep];//average kept
	//for(int irep=0;irep<Ed.size();irep++) Ed[irep]=0;//reset Dem Energy
	return EdInt;
/*	SU2 Uprime, Up,Ustaple;
//	int count=0,all=0;
	double EdInt=0;
	for (int LinCoord=0;LinCoord < Lat->Vol;LinCoord++) {
		for (int mu=0;mu<Nd;mu++) {
			Uprime=RMatrix();
			Ustaple=Lat->Staples(LinCoord,mu);
			Up=(*Lat->Umu(LinCoord,  mu))*Ustaple;
			double dE=-0.5*((Uprime*Up).trace()-(Up).trace());
			if (dE<Ed) Ed-=dE;
			EdInt+=Ed;
		}
	}
	EdInt/=(Lat->Vol*Nd);
//	return Ed;
	return EdInt;
	//FIX IT LIKE MIXED !!!!!!!!!!!!!!!!*/
}

SU2 Demon::RMatrix()
{
	SU2 RM;
	Vec4 aVec;
	aVec[0]=1.0-2*Lat->prnd->get();
	Lat->RandomSphere(sqrt(1-aVec[0]*aVec[0]),aVec[1],aVec[2],aVec[3]);
	RM(aVec[0],aVec[1],aVec[2],aVec[3]);
	return RM;
}

Demon::~Demon()
{
	delete Lat;
}
