#include "SU2Mixed.h"

SU2Mixed::SU2Mixed(Lattice &MyLat)
	: SU2Wilson(MyLat), OutAcceptFile("met_acceptance.d",ios::out)
{
}

double SU2Mixed::Action(SU2& Umu, int LinCoord, int mu) {
	SU2 Vmu;
	double act=0.0;
	for(int nu=0; (nu < Nd); nu++) {
		if(nu==mu) continue;
		for (int dir=-1;dir<=1;dir+=2) {
			Vmu=Umu*Lat->Staple(LinCoord,mu,nu,dir);
//decouple tr from trr(1) to make it faster!!
			for(int j=1;j<=Lat->Nrep;j++) 
				act+=Lat->Beta[j-1]*(1-Vmu.Trr(j)/(j+1));
		}
	}
	return act;
}

double SU2Mixed::Action0(SU2& Umu, int LinCoord, int mu) {
	SU2 Vmu;
	double act=0.0;
	for(int nu=0; (nu < Nd); nu++) {
		if(nu==mu) continue;
		for (int dir=-1;dir<=1;dir+=2) {
			Vmu=Umu*Lat->Staple(LinCoord,mu,nu,dir);
			act+=Lat->Betap*(1-Vmu.trace()/2);
		}
	}
	return act;
}

void SU2Mixed::Evolve(int NmOR)
{
	//NmOR ignored here
	int acceptance=0;
	for (int LinCoord=0; LinCoord<Lat->Vol; LinCoord++) {
		for (int mu=0;mu<Nd;mu++) {
			SU2 Uprime=HBtry(LinCoord,mu);
			SU2 Umu=*Lat->Umu(LinCoord,  mu);
			double xr=Lat->prnd->get();
			if(xr<exp(-Action(Uprime,LinCoord,mu)+Action0(Uprime,LinCoord,mu)
			  +Action(Umu,LinCoord,mu)-Action0(Umu,LinCoord,mu))) {
					*Lat->Umu(LinCoord,  mu)=Uprime;
					acceptance++;
			  }
		}
	}
	OutAcceptFile<<acceptance*1.0/(Lat->Vol*Nd)<<endl;
}

void SU2Mixed::EvolveSlice(int tin, int tfin)
{
	LatVec crd;
	int lcrd;
	for(crd[2]=0;crd[2]<Lat->lz;crd[2]++) {
		for(crd[1]=0;crd[1]<Lat->ly;crd[1]++) {
			for(crd[0]=0;crd[0]<Lat->lx;crd[0]++) {
				for(crd[3]=tin+1;crd[3]<tfin;crd[3]++) {
					Lat->Vec2Lin(crd,lcrd);
					for (int mu=0;mu<Nd;mu++) {
						SU2 Uprime=HBtry(lcrd,mu);
						SU2 Umu=*Lat->Umu(lcrd,  mu);
						double xr=Lat->prnd->get();
						if(xr<exp(-Action(Uprime,lcrd,mu)+Action0(Uprime,lcrd,mu)
			  			+Action(Umu,lcrd,mu)-Action0(Umu,lcrd,mu))) {
							*Lat->Umu(lcrd,  mu)=Uprime;
			  			}
					}
				}
				crd[3]=tin;
				Lat->Vec2Lin(crd,lcrd);
				SU2 Uprime=HBtry(lcrd,3);
				SU2 Umu=*Lat->Umu(lcrd,  3);
				double xr=Lat->prnd->get();
				if(xr<exp(-Action(Uprime,lcrd,3)+Action0(Uprime,lcrd,3)
			  		+Action(Umu,lcrd,3)-Action0(Umu,lcrd,3))) {
						*Lat->Umu(lcrd,  3)=Uprime;
			  	}
				
			}
		}
	}
}

void SU2Mixed::EvolveMetro()
{
	int acceptance=0;
	for (int LinCoord=0; LinCoord<Lat->Vol; LinCoord++) {
		for (int mu=0;mu<Nd;mu++) {

			SU2 Umu=*Lat->Umu(LinCoord,  mu);
			double xr=Lat->prnd->get();
			int hits=0;
			while (hits<10) {
				SU2 Uprime=RMatrix();
				if(xr<exp(-Action(Uprime,LinCoord,mu) +Action(Umu,LinCoord,mu))) {
					*Lat->Umu(LinCoord,  mu)=Uprime;
					acceptance++;
					hits=10;
				}
				hits++;
			}
		}
	}
	OutAcceptFile<<acceptance*1.0/(Lat->Vol*Nd)<<endl;
}

void SU2Mixed::Equilibrium(int EqSweeps)
{
//	if(Lat->MyState != STARTFILE) {
		int Eskip=1;
		if(EqSweeps>10) Eskip=EqSweeps/10;  
		cout<<"Equilibrization: ";
		for (int sweeps=0; sweeps<EqSweeps; sweeps++) {
			if(sweeps%Eskip == 0) 
				cout<<sweeps<<" "<<flush;
			Evolve();
		}
		cout<<EqSweeps<<endl;
//	}
}

SU2 SU2Mixed::RMatrix()
{
	SU2 RM;
	Vec4 aVec;
	aVec[0]=1.0-2*Lat->prnd->get();
	Lat->RandomSphere(sqrt(1-aVec[0]*aVec[0]),aVec[1],aVec[2],aVec[3]);
	RM(aVec[0],aVec[1],aVec[2],aVec[3]);
	return RM;
}
SU2Mixed::~SU2Mixed()
{}
