#include "meas.h"

Meas::Meas(Lattice &MyLat)
{
	Lat=&MyLat;
}

vector<double> Meas::PlaqAll(int MaxRep2)
{
	vector<double> plaqs;
	for(int i=1;i<=MaxRep2;i++) {
		plaqs.push_back(1-Plaq(i)/(i+1));
	}
	return plaqs;
}

double Meas::Plaq(int rep2) 
{
	LatVec ShiftVec;
	double plq=0;
	int LinCoord_mu;
	for (int LinCoord =0; LinCoord<Lat->Vol; LinCoord++) {
		for (int mu=1;mu<Nd;mu++) {
			ShiftVec[0]=0;ShiftVec[1]=0;ShiftVec[2]=0;
			ShiftVec[3]=0;ShiftVec[mu]=1;
			LinCoord_mu=Lat->CoordShift(LinCoord,  ShiftVec);
			for (int nu=0;nu<mu;nu++) {
				ShiftVec[0]=0;ShiftVec[1]=0;ShiftVec[2]=0;ShiftVec[3]=0;
				ShiftVec[nu]=1;
				plq+=((*Lat->Umu(LinCoord,  mu))*
				(*Lat->Umu(LinCoord_mu,  nu))*
				(*Lat->Umu(Lat->CoordShift(LinCoord, ShiftVec), mu)).dagger()*
				(*Lat->Umu(LinCoord,  nu)).dagger()).Trr(rep2);
			}
		}
	}
	return (2*plq/(Lat->Vol*Nd*(Nd-1))); //1/(V*Nd*(Nd-1)/2)
}

vector<double> Meas::WloopAll(int Length)
{
	vector<double> wloops;
	for(int cLen=2; cLen<=Length; cLen++) {
		wloops.push_back(1-Wloop(cLen)/2);
	}
	return wloops;
}

double Meas::Wloop(int cLen)
{// from "2 x 2" to "cLen x cLen" Wilson Loops
	LatVec ShiftVec;
	int LinCoord_mu;
	double plq=0;
	for (int LinCoord = 0; LinCoord < Lat->Vol; LinCoord++) {
		for (int mu=1;mu<Nd;mu++) {
			SU2 Ut(Lat->LongLink(LinCoord,mu,cLen));
			ShiftVec[0]=0;ShiftVec[1]=0;ShiftVec[2]=0;
			ShiftVec[3]=0;ShiftVec[mu]=cLen;
			LinCoord_mu=Lat->CoordShift(LinCoord,  ShiftVec);
			for (int nu=0;nu<mu;nu++) {
				ShiftVec[0]=0;ShiftVec[1]=0;ShiftVec[2]=0;ShiftVec[3]=0;
				ShiftVec[nu]=cLen;
				plq+=(Ut*(Lat->LongLink(LinCoord_mu,nu,cLen))*
				(Lat->LongLink(Lat->CoordShift(LinCoord,ShiftVec),mu,cLen)).dagger()*
				(Lat->LongLink(LinCoord,nu,cLen)).dagger()).trace();
			}
		}
	}
	return (2*plq/(Lat->Vol*Nd*(Nd-1)));//plq/(V*Nd*(Nd-1)/2)
}

double Meas::Ploop(int tin, int tfi)
{
	if(tfi==-1) tfi=Lat->lt;
	assert(tin<tfi);
	double PolyAverage=0;
	LatVec crd;
	int lcrd;
	
	for(crd[2]=0;crd[2]<Lat->lx;crd[2]++) {
		for(crd[1]=0;crd[1]<Lat->ly;crd[1]++) {
			for(crd[0]=0;crd[0]<Lat->lz;crd[0]++) {
				SU2 pLoop(1);
				for(crd[3]=tin;crd[3]<tfi;crd[3]++) {
					Lat->Vec2Lin(crd,lcrd);
					pLoop*=(*Lat->Umu(lcrd,3));
				}
				PolyAverage+=pLoop.trace();
			}
		}
	}
	return PolyAverage/(Lat->Vol)*(Lat->lt);
}

const SU2 Meas::PloopXYZ(LatVec Gcrd, int tin, int tfi)
{
	if(tfi==-1) tfi=Lat->lt;
	assert(tin<tfi);
	LatVec crd;
	int lcrd;
	for(int i=0;i<Nd;i++) crd[i]=Gcrd[i];
	SU2 pLoop(1);
	for(crd[3]=tin;crd[3]<tfi;crd[3]++) {
		Lat->Vec2Lin(crd,lcrd);
		pLoop*=(*Lat->Umu(lcrd,3));
	}
	return pLoop;
}
void Meas::PloopCor(int rMax, vector<double> *Ploop) {
	const int V3=(Lat->lx)*(Lat->ly)*(Lat->lz);
	SU2 P1, P2;
	LatVec crd,crd_r; crd[3]=crd_r[3]=0; //space volume coord at zero time slice
	//cycle over space volume 
	for(crd[2]=0;crd[2]<Lat->lz;crd[2]++) {
		for(crd[1]=0;crd[1]<Lat->ly;crd[1]++) {
			for(crd[0]=0;crd[0]<Lat->lx;crd[0]++) {
//				int lcrd=crd[0]+Lat->lx*(crd[1]+Lat->ly*crd[2]);
				P1=PloopXYZ(crd);
				(*Ploop)[0]+=((P1.dagger()).trace()*P1.trace());
				for (int r=1;r<=rMax;r++) {
					crd_r[0]=crd[0]+r; crd_r[1]=crd[1]; crd_r[2]=crd[2];
					P2=PloopXYZ(crd_r);
					(*Ploop)[r]+=((P1.dagger()).trace()*P2.trace());
				}
			}
		}
	}
	for(int r=0;r<=rMax;r++) (*Ploop)[r]/=V3;
}

void Meas::PWloopCorLW(const int rMax,SU2Wilson & MyModel, vector <double> *Ploop, vector <double> *Wloop) {
	int Nslice=100;
	
	LatVec crd,crd_r; crd[3]=crd_r[3]=0; //space volume coord at zero time slice
	const int V3=(Lat->lx)*(Lat->ly)*(Lat->lz);
	const int SlWidth=2;
	SU2 P1, P2;
	SU2xSU2 **Tall = new SU2xSU2* [V3];
	for (int i=0;i<V3;i++) {
		Tall[i] = new SU2xSU2[rMax+1];
	}
	SU2xSU2 **Tslice = new SU2xSU2* [V3];
	for (int i=0;i<V3;i++) {
		Tslice[i] = new SU2xSU2[rMax+1];
	}
	SU2xSU2 Ttemp;
	for(int tslice=0;tslice<Lat->lt;tslice+=SlWidth) {
		for(int i=0;i<V3;i++) {
				for (int r=0;r<=rMax;r++) Tslice[i][r](0.0);
		}
		for(int SliceSweep=0;SliceSweep<Nslice;SliceSweep++) {
			if(SliceSweep>0) MyModel.EvolveSlice(tslice,tslice+SlWidth);
			//cycle over space volume 
			for(crd[2]=0;crd[2]<Lat->lz;crd[2]++) {
				for(crd[1]=0;crd[1]<Lat->ly;crd[1]++) {
					for(crd[0]=0;crd[0]<Lat->lx;crd[0]++) {
						int lcrd=crd[0]+Lat->lx*(crd[1]+Lat->ly*crd[2]);//change to inline function
						P1=PloopXYZ(crd,tslice,tslice+SlWidth);
						Ttemp.DirectProd(P1,P1);
						Tslice[lcrd][0]+=Ttemp;
						for (int r=1;r<=rMax;r++) {
							crd_r[0]=crd[0]+r; crd_r[1]=crd[1]; crd_r[2]=crd[2];
							P2=PloopXYZ(crd_r,tslice,tslice+SlWidth);
							Ttemp.DirectProd(P1,P2);
							Tslice[lcrd][r]+=Ttemp;
						}
					}
				}
			}
		}
		if(tslice==0) {
			for(int i=0;i<V3;i++) {
				for(int r=0;r<=rMax;r++) {
					Tslice[i][r]*=(1.0/Nslice);
					Tall[i][r]=Tslice[i][r];
				}
			}
		} else {
			for(int i=0;i<V3;i++) {
				for(int r=0;r<=rMax;r++) {
					Tslice[i][r]*=(1.0/Nslice);
					Tall[i][r]*=Tslice[i][r];
				}
			}
		}
		/*if(Wloop && tslice==Lat->lt-2*SlWidth) {//the lattice size in T direction - slice
			for(int r=0;r<=rMax;r++) {
				for(int i=0;i<V3;i++) {
					// Wilson loop part
					if(r>0) {
						LatVec tcrd;
						Lat->Lin2Vec(Lat->lt*i, tcrd);
						tcrd[3]+=tslice+SlWidth;
						int tlin;
						Lat->Vec2Lin(tcrd,tlin);
						SU2 L0(Lat->LongLink(Lat->lt*i,0,r));
						SU2 L1(Lat->LongLink(tlin,0,r));
						std::complex<double> OneLoop(Tall[i][r].Tloop(L0,L1.dagger()));
						(*Wloop)[r]+=real(OneLoop);
					} else if (r==0) {
						SU2 Unit(1.0);
						std::complex<double> OneLoop(Tall[i][r].Tloop(Unit,Unit));
						(*Wloop)[r]+=real(OneLoop);
					}
				}
				(*Wloop)[r]/=V3;
			}
		}*/
	}
	//clean temp 
	for (int i=0;i<V3;i++) {
		delete [] Tslice[i];
	}
	delete [] Tslice;
	for(int r=0;r<=rMax;r++) {
		for(int i=0;i<V3;i++) {
			// Polyakov loop part
			(*Ploop)[r]+=Tall[i][r].trace();
			if(Wloop) {
				// Wilson loop part
				if(r>0) {
					SU2 L0(Lat->LongLink(Lat->lt*i,0,r));
					std::complex<double> OneLoop(Tall[i][r].Tloop(L0,L0.dagger()));
					(*Wloop)[r]+=real(OneLoop);
				} else if (r==0) {
					SU2 Unit(1.0);
					std::complex<double> OneLoop(Tall[i][r].Tloop(Unit,Unit));
					(*Wloop)[r]+=real(OneLoop);
				}
			}
		}
		(*Ploop)[r]/=V3;
		if (Wloop) (*Wloop)[r]/=V3;
	}
	//clean up 
	//delete [] Tall;
	for (int i=0;i<V3;i++) {
		delete [] Tall[i];
	}
	delete [] Tall;
}

Meas::~Meas()
{}
