#include "parameters.h"
Params::Params(const char * ParFileName)
{
	ifstream inFile(ParFileName, ios::in);
	if(!inFile) {
		std::cerr<<"Cannot open parameter file"<<std::endl;
		abort();
	}
	string line;
	//Defaults:
	Evol=Equil=OverRel=0;
	start=ORDERED;
	Nrep=MeasSkip=1;
	WloopLength=0;
	//---------
	while (getline(inFile,line,'\n'))
	{
		if(line.substr(0,1)=="#" || line.size()==0) continue; //skip comment line
		istringstream inputString(line);
		string ParName;
		inputString>>ParName;
		
		vector<double> inDat;
		double inTemp;
		while(inputString>>inTemp) inDat.push_back(inTemp);
		vector<double>::const_iterator pv;
		cout<<ParName<<" ";
		for(pv=inDat.begin();pv!=inDat.end(); pv++) cout<<*pv<<" ";
		cout<<endl;
		if (ParName=="Beta") {
			Beta=inDat;
			Nrep=Beta.size();
			cout<<"Number of representations: "<<Nrep<<endl;
		}
		else if (ParName=="Betap") {
			Betap=inDat[0];
		} 
		else if (ParName=="LatSize"){
			if(inDat.size()!=4) {
				cerr<<"Only 4d lattices are supported"<<endl;
				abort();
			}
			int iiter=0;
			for(pv=inDat.begin();pv!=inDat.end(); pv++) {
				LatSize[iiter++]=static_cast<int>(*pv);
			}
		}
		else if (ParName=="Start") {
			start=static_cast<enum State>(inDat[0]);
			if(start==ORDERED) cout<<"(Ordered start)"<<endl;
			else if(start==RANDOM) cout<<"(Random start)"<<endl;
			else if(start==STARTFILE) cout<<"(Start from a file)"<<endl;
			else {
				cout<<"Wrong start parameter: "<<start
				  <<" (Start range 0..2)"<<endl;
				abort();
			}
		}
		else if(ParName=="Equil") {
			Equil=static_cast<int> (inDat[0]);
		}
		else if(ParName=="Evol") {
			Evol=static_cast<int> (inDat[0]);
		}
		else if(ParName=="Iseed") {
			Iseed=static_cast<int> (inDat[0]);
		}
		else if(ParName=="MeasSkip") {
			MeasSkip=static_cast<int>(inDat[0]);
		}
		else if(ParName=="OverRel") {
			OverRel=static_cast<int>(inDat[0]);
		}
		else if(ParName=="Plaq") ToMeasure.insert(MOentry (Plaq, ParName));
		else if(ParName=="Ploop") ToMeasure.insert(MOentry (Ploop, ParName));
		else if(ParName=="Wloop") {
			ToMeasure.insert(MOentry (Wloop, ParName));
			WloopLength=static_cast<int>(inDat[0]);
		}
		else if(ParName=="PloopCor") ToMeasure.insert(MOentry (PloopCor, ParName));
		else if(ParName=="WloopLarge") ToMeasure.insert(MOentry (WloopLarge, ParName));
		else if(ParName=="DemonEn") {
			ToMeasure.insert(MOentry (DemonEn, ParName));
			DemonSweeps=static_cast<int>(inDat[0]);
		}
		else if(ParName=="Decimation") {
			Decimation=static_cast<int>(inDat[0]);
			DecSWc=inDat[1];
		}
		else {
			cerr<<"Unknown input parameter: "<<ParName<<endl;
		}
	}
	cout<<"Will measure: ";
	if (ToMeasure.count(Plaq)) cout<<"Plaquettes";
	if (ToMeasure.count(Wloop)) cout<<", Wilson loops (<="<<WloopLength<<")";
	if (ToMeasure.count(Ploop)) cout<<", Polyakov loops";
	if (ToMeasure.count(PloopCor)) cout<<", Polyakov loop correlators";
	if (ToMeasure.count(DemonEn)) cout<<", Demon energies";
	if (Decimation==1) {
		cout<<" ... and perform Swendsen type decimation\n";
		cout<<"(with staples weight "<<DecSWc<<")";
	}
	if (Decimation==2) {
		cout<<" ... and perform Double Smeared Blocking";
		cout<<"(with staples weight "<<DecSWc<<")";
	}
	cout<<"."<<endl;
}

void Params::DecReinit() {
	for(int i=0;i<Nd;i++) {
		LatSize[i]/=2;
	}
	Nrep=8;
}
